"""Script for running a tsunami inundation scenario for Onslow, WA, Australia.

Source data such as elevation and boundary data is assumed to be available in
directories specified by project.py
The output sww file is stored in project.outputtimedir 

The scenario is defined by a triangular mesh created from project.polygon,
the elevation data and a tsunami generated by a subduction zone earthquake.

Ole Nielsen and Duncan Gray, GA - 2005 and Nick Bartzis, GA - 2006
"""
#-------------------------------------------------------------------------------# Import necessary modules
#-------------------------------------------------------------------------------

# Standard modules
from os import sep
from os.path import dirname, basename
import time

# Related major packages
from anuga.shallow_water import Domain, Reflective_boundary, \
                            Dirichlet_boundary, Time_boundary, File_boundary
from anuga.shallow_water.data_manager import convert_dem_from_ascii2netcdf, \
     dem2pts
from anuga.abstract_2d_finite_volumes.combine_pts import combine_rectangular_points_files 
from shutil import copy
from os import mkdir, access, F_OK
from anuga.geospatial_data.geospatial_data import *
import sys
from anuga.abstract_2d_finite_volumes.util import start_screen_catcher, copy_code_files

# Application specific imports
import project                 # Definition of file names and polygons

#-------------------------------------------------------------------------------
# Copy scripts to time stamped output directory and capture screen
# output to file
#-------------------------------------------------------------------------------

copy_code_files(project.outputtimedir,__file__,dirname(project.__file__)+sep+ project.__name__+'.py' )
myid = 0
numprocs = 1
start_screen_catcher(project.outputtimedir, myid, numprocs)

print 'USER:    ', project.user

#-------------------------------------------------------------------------------
# Preparation of topographic data
#
# Convert ASC 2 DEM 2 PTS using source data and store result in source data
# Do for coarse and fine data
# Fine pts file to be clipped to area of interest
#-------------------------------------------------------------------------------

# filenames
meshname = project.meshname+'.msh'
source_dir = project.boundarydir

# fine data (clipping the points file to smaller area)
# creates DEM from asc data 
convert_dem_from_ascii2netcdf(project.onshore_dem_name, use_cache=True, verbose=True)

#creates pts file from DEM
dem2pts(project.onshore_dem_name,
        easting_min=project.eastingmin,
        easting_max=project.eastingmax,
        northing_min=project.northingmin,
        northing_max= project.northingmax,
        use_cache=True,
        verbose=True)

print 'create offshore'
G11= Geospatial_data(file_name = project.offshore_dem_name0 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name1 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name2 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name3 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name4 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name5 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name6 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name7 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name8 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name9 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name10 + '.xya')
G12= Geospatial_data(file_name = project.offshore_dem_name11 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name12 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name13 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name14 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name15 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name16 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name17 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name18 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name19 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name20 + '.xya')
G13= Geospatial_data(file_name = project.offshore_dem_name21 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name22 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name23 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name24 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name25 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name26 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name27 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name28 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name29 + '.xya')
G14= Geospatial_data(file_name = project.offshore_dem_name30 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name31 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name32 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name33 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name34 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name35 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name36 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name37 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name38 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name39 + '.xya')
G15= Geospatial_data(file_name = project.offshore_dem_name40 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name41 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name42 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name43 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name44 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name45 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name46 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name47 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name48 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name49 + '.xya')+\
     Geospatial_data(file_name = project.offshore_dem_name50 + '.xya')
   
print 'create onshore'
G2 = Geospatial_data(file_name = project.onshore_dem_name + '.pts')
print 'create coast'
G3 = Geospatial_data(file_name = project.coast_dem_name + '.xya')
print 'add'
G = G11 + G12 + G13 + G14 + G15 + G2 + G3
print 'export G'
G.export_points_file(project.combined_dem_name + '.pts')

#-------------------------------------------------------------------------------                                 
# Create the triangular mesh based on overall clipping polygon with a tagged
# boundary and interior regions defined in project.py along with
# resolutions (maximal area of per triangle) for each polygon
#-------------------------------------------------------------------------------

from anuga.pmesh.mesh_interface import create_mesh_from_regions

region_res = 500000
coast_res = 500
pt_hedland_res = 5000
interior_regions = [[project.poly_pt_hedland, pt_hedland_res],
                    [project.poly_region, region_res]]

print 'number of interior regions', len(interior_regions)

from anuga.utilities.polygon import plot_polygons
if sys.platform == 'win32':
    #figname = project.outputtimedir + 'pt_hedland_polys'
    figname = 'pt_hedland_polys_test'
    plot_polygons([project.polyAll,project.poly_pt_hedland,project.poly_region],
              figname,
              verbose = True)    

print 'start create mesh from regions'
from caching import cache
_ = cache(create_mesh_from_regions,
          project.polyAll,
          {'boundary_tags': {'topright': [0], 'topleft': [1],
                             'left': [2], 'bottom0': [3],
                             'bottom1': [4], 'bottom2': [5],
			     'bottom3': [6], 'right': [7]},
           'maximum_triangle_area': 250000,
           'filename': meshname,           
           'interior_regions': interior_regions},
          verbose = True, evaluate=True)

#-------------------------------------------------------------------------------                                 
# Setup computational domain
#-------------------------------------------------------------------------------                                 
domain = Domain(meshname, use_cache = False, verbose = True)

print domain.statistics()
print 'Number of triangles = ', len(domain)
print 'The extent is ', domain.get_extent()
print domain.statistics()

domain.set_name(project.basename)
domain.set_datadir(project.outputtimedir)
domain.set_quantities_to_be_stored(['stage', 'xmomentum', 'ymomentum'])

#-------------------------------------------------------------------------------                                 
# Setup initial conditions
#-------------------------------------------------------------------------------

tide = 0.
#high
#tide = 3.6
#low
#tide = -3.9

domain.set_quantity('stage', tide)
domain.set_quantity('friction', 0.0) 
print 'hi and file',project.combined_dem_name + '.pts'

domain.set_quantity('elevation', 
                    filename = project.combined_dem_name + '.pts',
                    use_cache = True,
                    verbose = True,
                    alpha = 0.1
                    )

#-------------------------------------------------------------------------------                                 
# Setup boundary conditions (all reflective)
#-------------------------------------------------------------------------------
print 'start ferret2sww'
# skipped as results in file SU-AU_clipped is correct for all WA

from anuga.pyvolution.data_manager import ferret2sww

south = project.south 
north = project.north
west = project.west
east = project.east

#note only need to do when an SWW file for the MOST boundary doesn't exist
cache(ferret2sww,
      (source_dir + project.boundary_basename,
       source_dir + project.boundary_basename+'_'+project.basename), 
      {'verbose': True,
       'minlat': south,
       'maxlat': north,
       'minlon': west,
       'maxlon': east,
#       'origin': project.mesh_origin,
       'origin': domain.geo_reference.get_origin(),
       'mean_stage': tide,
       'zscale': 1,                 #Enhance tsunami
       'fail_on_NaN': False,
       'inverted_bathymetry': True},
       evaluate = True,
       verbose = True,
      dependencies = source_dir + project.boundary_basename + '.sww')

print 'Available boundary tags', domain.get_boundary_tags()

Bf = File_boundary(source_dir + project.boundary_basename + '.sww', 
                    domain, verbose = True)
Br = Reflective_boundary(domain)
Bd = Dirichlet_boundary([tide,0,0])
domain.set_boundary( {'topright': Bf,'topleft': Bf, 'left':  Bd, 'bottom0': Bd,
                      'bottom1': Bd, 'bottom2': Bd, 'bottom3': Bd, 
			'right': Bd})

#-------------------------------------------------------------------------------                                 
# Evolve system through time
#-------------------------------------------------------------------------------
import time
t0 = time.time()

for t in domain.evolve(yieldstep = 240, finaltime = 10800): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'topright')     

for t in domain.evolve(yieldstep = 120, finaltime = 16200
                       ,skip_initial_step = True): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'topright')     

for t in domain.evolve(yieldstep = 60, finaltime = 21600
                       ,skip_initial_step = True): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'topright')     
    
for t in domain.evolve(yieldstep = 120, finaltime = 27000
                       ,skip_initial_step = True): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'topright')     

for t in domain.evolve(yieldstep = 240, finaltime = 36000
                       ,skip_initial_step = True): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'topright')   
  
print 'That took %.2f seconds' %(time.time()-t0)

print 'finished'
